/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.commons.database;

import org.apache.commons.configuration.Configuration;
import org.hibernate.dialect.DB2Dialect;
import org.hibernate.dialect.HSQLDialect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonar.commons.database.dialect.*;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.PersistenceException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Properties;

public abstract class AbstractDatabaseConnector implements DatabaseConnector {
  protected static final Logger LOG_SQL = LoggerFactory.getLogger("org.hibernate.SQL");
  protected static final Logger LOG_STATISTICS = LoggerFactory.getLogger("org.sonar.DBSTATISTICS");

  private Configuration configuration = null;
  private EntityManagerFactory factory = null;
  private int databaseVersion = SchemaMigration.VERSION_UNKNOWN;
  private boolean operational = false;
  private boolean started = false;
  private boolean startsFailIfSchemaOutdated;
  private Integer transactionIsolation = null;

  protected AbstractDatabaseConnector(Configuration configuration, boolean startsFailIfSchemaOutdated) {
    this.configuration = configuration;
    this.startsFailIfSchemaOutdated = startsFailIfSchemaOutdated;
  }

  protected AbstractDatabaseConnector() {
  }

  public Configuration getConfiguration() {
    return configuration;
  }

  public void setConfiguration(Configuration configuration) {
    this.configuration = configuration;
  }

  /**
   * Indicates if the connector is operational : database connection OK and schema version OK
   */
  public boolean isOperational() {
    return operational;
  }

  /**
   * Indicates if the connector is started : database connection OK and schema version OK or KO
   */
  protected boolean isStarted() {
    return started;
  }

  /**
   * Get the JDBC transaction isolation defined by the configuration
   *
   * @return JDBC transaction isolation
   */
  public Integer getTransactionIsolation() {
    return transactionIsolation;
  }

  public void start() {
    if (!started) {
      transactionIsolation = configuration.getInteger(DatabaseProperties.PROP_ISOLATION, null /* use driver default setting */);
      testConnection();
      started = true;
    }
    if (!operational) {
      boolean upToDate = upToDateSchemaVersion();
      if (!upToDate && startsFailIfSchemaOutdated) {
        throw new DatabaseException(databaseVersion, SchemaMigration.LAST_VERSION);
      }
      if (upToDate) {
        factory = createEntityManagerFactory();
        operational = true;
      }
    }
  }

  public void stop() {
    if (factory != null && factory.isOpen()) {
      factory.close();
      factory = null;
    }
    operational = false;
    started = false;
  }

  public abstract void setupEntityManagerFactory(Properties factoryProps);

  public EntityManagerFactory getEntityManagerFactory() {
    return factory;
  }

  protected void setEntityManagerFactory(EntityManagerFactory factory) {
    this.factory = factory;
  }

  protected EntityManagerFactory createEntityManagerFactory() {
    // other settings are stored into /META-INF/persistence.xml
    Properties props = new Properties();
    if (transactionIsolation != null) {
      props.put("hibernate.connection.isolation", Integer.toString(transactionIsolation));
    }
    props.put("hibernate.hbm2ddl.auto", getConfiguration().getString(DatabaseProperties.PROP_HIBERNATE_HBM2DLL, "validate"));
    props.put("hibernate.dialect", getDialectClass(getDialect()));

    props.put("hibernate.generate_statistics", new Boolean(LOG_STATISTICS.isInfoEnabled()).toString());
    props.put("hibernate.show_sql", new Boolean(LOG_SQL.isInfoEnabled()).toString());

    if (getConfiguration().getBoolean(DatabaseProperties.PROP_HIBERNATE_SECOND_LEVEL_CACHE, true)) {
      props.put("hibernate.cache.provider_class", "org.hibernate.cache.EhCacheProvider");
      props.put("hibernate.cache.use_second_level_cache", "true");
      props.put("hibernate.cache.use_query_cache", "false");

    } else {
      props.put("hibernate.cache.provider_class", "org.hibernate.cache.NoCacheProvider");
      props.put("hibernate.cache.use_second_level_cache", "false");
      props.put("hibernate.cache.use_query_cache", "false");
    }
    // custom impl setup
    setupEntityManagerFactory(props);
    return Persistence.createEntityManagerFactory("sonar", props);
  }

  public EntityManager createEntityManager() {
    EntityManager manager = factory.createEntityManager();
    if (LOG_STATISTICS.isInfoEnabled()) {
      manager = new StatisticsEntityManager(factory, manager, LOG_STATISTICS);
    }
    return manager;
  }

  private void testConnection() throws DatabaseException {
    Connection connection = null;
    try {
      connection = getConnection();
    } catch (SQLException e) {
      throw new DatabaseException("Cannot open connection to database: " + e.getMessage(), e);
    } finally {
      silentlyCloseConnection(connection);
    }
  }

  protected int loadVersion() {
    Connection connection = null;
    try {
      connection = getConnection();
      return SchemaMigration.getCurrentVersion(connection);
    } catch (SQLException e) {
      // schema not created
      return 0;
    } finally {
      silentlyCloseConnection(connection);
    }
  }

  private void silentlyCloseConnection(Connection connection) {
    if (connection != null) {
      try {
        connection.close();
      } catch (SQLException e) {
      }
    }
  }

  private boolean upToDateSchemaVersion() {
    if (databaseVersion == SchemaMigration.LAST_VERSION) {
      return true;
    }
    databaseVersion = loadVersion();
    return databaseVersion == SchemaMigration.LAST_VERSION;
  }

  protected int getDatabaseVersion() {
    return databaseVersion;
  }

  public String getDialect() {
    String dialect = configuration.getString("sonar.jdbc.dialect");
    if (dialect == null) {
      Connection connection = null;
      try {
        connection = getConnection();
        dialect = getDialectFromJdbcUrl(connection.getMetaData().getURL());

      } catch (SQLException e) {
        throw new PersistenceException("can not autodetect the dialect", e);

      } finally {
        silentlyCloseConnection(connection);
      }
    }
    return dialect;
  }

  public String getDialectClass(String dialect) {
    String dialectClass = configuration.getString(DatabaseProperties.PROP_DIALECT_CLASS);
    if (dialectClass == null) {
      dialectClass = getHibernateDialectClassName(dialect);
    }
    return dialectClass;
  }

  private String getDialectFromJdbcUrl(String url) {
    if (url.toLowerCase().startsWith("jdbc:db2:")) {
      return DatabaseProperties.DIALECT_DB2;
    }
    if (url.toLowerCase().startsWith("jdbc:derby:")) {
      return DatabaseProperties.DIALECT_DERBY;
    }
    if (url.toLowerCase().startsWith("jdbc:hsqldb:")) {
      return DatabaseProperties.DIALECT_HSQLDB;
    }
    if (url.toLowerCase().startsWith("jdbc:microsoft:sqlserver:") ||
        url.toLowerCase().startsWith("jdbc:jtds:sqlserver:")) {
      return DatabaseProperties.DIALECT_MSSQL;
    }
    if (url.toLowerCase().startsWith("jdbc:mysql:")) {
      return DatabaseProperties.DIALECT_MYSQL;
    }
    if (url.toLowerCase().startsWith("jdbc:oracle:")) {
      return DatabaseProperties.DIALECT_ORACLE;
    }
    if (url.toLowerCase().startsWith("jdbc:postgresql:")) {
      return DatabaseProperties.DIALECT_POSTGRESQL;
    }
    return null;
  }

  private String getHibernateDialectClassName(String dialect) {
    if (DatabaseProperties.DIALECT_DB2.equals(dialect)) {
      return DB2Dialect.class.getName();
    }
    if (DatabaseProperties.DIALECT_DERBY.equals(dialect)) {
      return DerbyWithDecimalDialect.class.getName();
    }
    if (DatabaseProperties.DIALECT_HSQLDB.equals(dialect)) {
      return HSQLDialect.class.getName();
    }
    if (DatabaseProperties.DIALECT_MSSQL.equals(dialect)) {
      return MsSqlDialect.class.getName();
    }
    if (DatabaseProperties.DIALECT_MYSQL.equals(dialect)) {
      return MySqlWithDecimalDialect.class.getName();
    }
    if (DatabaseProperties.DIALECT_ORACLE.equals(dialect)) {
      return Oracle10gWithDecimalDialect.class.getName();
    }
    if (DatabaseProperties.DIALECT_POSTGRESQL.equals(dialect)) {
      return PostgreSQLWithDecimalDialect.class.getName();
    }
    return null;
  }
}
